#ifndef MOD_EVALUATE
#define MOD_EVALUATE

#include "ModGeometry.H"
#include "ModMaterial.H"
#include "ModControl.H"   // ParticleBox 

#include "PT/PTC_Para.h"
#include "PT/PT_Misc.h"
#include "PT/PTS_StatIndex.h"
#include "PT/PTS_Covariance.h"

// prototype(s) of the function that wil be called with the info in from 
// the 'save' files to do data evaluation. 

#if MOD_JANSEN_PRESSURE
// run over particles, divide pressure by surface and find maximum 
struct ModEvaluate_Jansen {
  static const int N_ITVLS = 30;

  ModEvaluate_Jansen( Material &a_mat, double max_height ) : 
    mat(a_mat), max_pressure(0.), map(0.,max_height,N_ITVLS) {}

  void operator()(ParticleBox::value_type &p){
    p.pressure /= 4.* M_PI * PTsqr(mat.getRadius(p.type));
    max_pressure = PTmax(max_pressure, p.pressure);
    pressure[ map(p.x[2]) ].sample( p.pressure );
  }
  Material &mat;
  double max_pressure;
  PTS_Covariance<PTZ_Vector<double,1>,1> pressure[N_ITVLS]; // pressure bins
  PTS_StatIndex_Lin<double>              map;               // height -> index
};
#endif


struct Evaluation {
  // this is the info that we actually get from the saved file
  // ReadSaveFile(istream &in, double &t, ParticleBox &box, PTC_Para &pf );

  // initialization - arguments not necessarily needed
  Evaluation( PTC_Para &, Geometry &, Material &);

  // is presented each configuration being read
  void 
  configuration(double t, ParticleBox &, PTC_Para &, Geometry &, Material &);

  ~Evaluation();

  double e_kin;  // set each time configuration is called
#if MOD_JANSEN_PRESSURE
  ModEvaluate_Jansen  jansen;
#endif
};

#endif












