
#include "ModEvaluate.H"


struct ModEvaluate_SumUp {
  ModEvaluate_SumUp( Material &a_mat ) : mat(a_mat), e_kin(0.) {}

  void operator()(ParticleBox::value_type &p){
    e_kin += .5 * mat.getMass(p.type) * dot(p.v,p.v);
    e_kin += .5 * mat.getMomentOfInertia(p.type) * dot(p.omega,p.omega);
  }
  Material &mat;
  double e_kin;
};



// takes stuff that we know before the first saved configuration is read 
Evaluation::Evaluation( PTC_Para &, Geometry &geo, Material &mat)
#if MOD_JANSEN_PRESSURE
  : jansen(mat,geo.ur[2]-geo.ll[2])
#endif
{}

void
Evaluation::configuration(double t, ParticleBox &box, 
			  PTC_Para &pf, Geometry &geo, Material &mat) {

  (void) t;  (void) pf;  (void) geo;   // make g++-3.0 happy

  // compute the kinetic and potential energy per particle
  // we cannot (for now) include the 
  // potential energy stored in the elastic friction 
  // "springs" or in the particle overlaps (need to read materials file)

  ModEvaluate_SumUp   energy(mat);
  box.for_each_particle( energy );

#if MOD_JANSEN_PRESSURE
  jansen.max_pressure = 0.;
  // jansen compute pressure, its maximum and bins according to height
  box.for_each_particle( jansen );
#endif
  
  // member variable
  e_kin = energy.e_kin; 
  if ( box.size() ) 
    e_kin /= box.size();
  cout << "E_kin/Part=" << e_kin;
}


Evaluation::~Evaluation(){ 
#if MOD_JANSEN_PRESSURE 
  cout << "# height    pressure\n"; 
  for ( int i=0; i < 30; ++i )
    cout << jansen.map.get_value_of_bin(i) << " " 
	 << jansen.pressure[i].get_mean() << "\n";
  cout << endl;
#endif
}






