#ifndef MOD_GEOMETRY_H
#define MOD_GEOMETRY_H

#include "IOFuncs.H"
#include "ModWall.H"

#include "PT/PTBinaryIO.h"
#include "PT/PTS_Factory.h"
#include "PT/PTC_Para.h"
#include "PT/PTDebug.h"
#include "ModControl.H"
#include "PT/PTS_CommBase.h"

// ---------------------------------------------------------
// Class Geometry - Container for Walls
// ---------------------------------------------------------

// not complete: should at least contain the functions to read
// a geomtery file (operator>>())

class Geometry {

public:
  /** select wall items (features) which might cause interaction with 
      particles in the specified volume, for subsequent use with op() */
  void select_features(double dist, const double_vec proc_ll, const double_vec proc_ur);

  /** negotiates global ids for the wall segments */
  void set_wallfeature_ids(Control &, PTS_CommEnv &);

  /** intended for processor local loops over all walls 
      (eg. for particle-wall interaction)
   */ 
  template<class P>
  void operator()(P &p){
    for ( vector<WallFeature *>::iterator it = localWalls.begin();
	  it != localWalls.end();
	  ++it )
      (**it)(p);  
  }
      
  int size();

  Geometry( istream &in );
  Geometry( PTC_Para &pf );
  
  void update( double t );

  std::ostream &save(std::ostream &out) const;
  std::istream &restore(std::istream &in);

  void ModIO_ascii_in(std::istream &in);
  void ModIO_ascii_out(PTS_CommEnv &ce, int root_node, std::ostream &out);
  void ModIO_bin_in(std::istream &in);
  void ModIO_bin_out(PTS_CommEnv &ce, int root_node, std::ostream &out);

#ifdef PTM_VISUALIZE
  // draws geometry
  void draw(XTDisplay &window);
#endif
 
public: // ughh - FIXME

  double_vec ll, ur;

private:
  // common ctor code
  void init( istream &in);

  vector<WallFeature *> allWalls;   // all features read from file
  // features that might interact with 
  // particles in the volume in the last call to select_features()
  vector<WallFeature *> localWalls; 

  // 'virtual' construction of objects from keywords
  static PTS_ExtTagFactory<WallFeature,std::string> our_factory;
};



#endif

